#region Using

using System;
using System.Web;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Net;

#endregion

/// <summary>
/// Summary description for SocialGraph
/// </summary>
public class SocialGraph
{

	#region Properties

	private bool _EnableOutboundLinks;
	/// <summary>
	/// Gets or sets a value indicating whether outbound links should be parsed.
	/// If outbound links are disabled, then verified claims cannot be parsed.
	/// </summary>
	/// <value><c>true</c> if outbound links should be parsed; otherwise, <c>false</c>.</value>
	public bool EnableOutboundLinks
	{
		get { return _EnableOutboundLinks; }
		set { _EnableOutboundLinks = value; }
	}

	private bool _EnableInboundLinks;
	/// <summary>
	/// Gets or sets a value indicating whether inbound links should be parsed.
	/// If inbound links are disabled, then verified claims cannot be parsed.
	/// </summary>
	/// <value><c>true</c> if inbound links should be parsed; otherwise, <c>false</c>.</value>
	public bool EnableInboundLinks
	{
		get { return _EnableInboundLinks; }
		set { _EnableInboundLinks = value; }
	}

	private Uri _Url;
	/// <summary>
	/// Gets or sets the URL of the webpage to analyze.
	/// </summary>
	/// <value>The URL of the webpage.</value>
	public Uri Url
	{
		get { return _Url; }
		set { _Url = value; }
	}

	private string _Html;
	/// <summary>
	/// Gets the HTML retrieved from the URL.
	/// </summary>
	/// <value>The HTML retrieved.</value>
	public string Html
	{
		get { return _Html; }
	}

	private StringCollection _VerifiedClaims = new StringCollection();
	/// <summary>
	/// Gets the verified claims. When a claim is verified it means that there is both
	/// an inbound AND outbound link pointing to each other with the rel="me" attribute.
	/// </summary>
	/// <value>The verified claims.</value>
	public StringCollection VerifiedClaims
	{
		get { return _VerifiedClaims; }
	}

	private StringCollection _UnverifiedClaims = new StringCollection();
	/// <summary>
	/// Gets the unverified claims. When a claim is unverified it means that there is only
	/// an inbound OR outbound link pointing to each other with the rel="me" attribute.
	/// </summary>
	/// <value>The verified claims.</value>
	public StringCollection UnverifiedClaims
	{
		get { return _UnverifiedClaims; }
	}

	private Dictionary<string, StringCollection> _Outbound = new Dictionary<string, StringCollection>();
	/// <summary>
	/// Gets the outbound links marked-up with XFN tags.
	/// </summary>
	/// <value>The outbound links.</value>
	public Dictionary<string, StringCollection> Outbound
	{
		get { return _Outbound; }
	}

	private Dictionary<string, StringCollection> _Inbound = new Dictionary<string, StringCollection>();
	/// <summary>
	/// Gets the inbound links marked-up with XFN tags.
	/// </summary>
	/// <value>The inbound links.</value>
	public Dictionary<string, StringCollection> Inbound
	{
		get { return _Inbound; }
	}

	#endregion

	#region Public methods

	/// <summary>
	/// Updates and parses the webpage of the specified URL.
	/// </summary>
	public void Update()
	{
		if (Url == null)
			throw new NotSupportedException("The Url must be set before updating");

		Update(Url);
	}

	/// <summary>
	/// Updates and parses the webpage of the specified URL.
	/// </summary>
	/// <param name="url">The URL of the webpage to analyze.</param>
	public void Update(Uri url)
	{
		Update(url, EnableOutboundLinks, EnableInboundLinks);
	}

	/// <summary>
	/// Updates and parses the webpage of the specified URL.
	/// </summary>
	/// <param name="url">The URL of the webpage to analyze.</param>
	/// <param name="enableOutboundLinks">if set to <c>true</c> outbound links will be parsed.</param>
	/// <param name="enableInboundLinks">if set to <c>true</c> inbound links will be parsed.</param>
	public void Update(Uri url, bool enableOutboundLinks, bool enableInboundLinks)
	{
		_Url = url;
		_EnableOutboundLinks = enableOutboundLinks;
		_EnableInboundLinks = enableInboundLinks;
		RetrieveHtml();

		FindVerifiedClaims();

		if (EnableOutboundLinks)
			FindOutboundLinks();

		if (EnableInboundLinks)
			FindInboundLinks();

		if (EnableInboundLinks && EnableOutboundLinks)
			FindUnverifiedClaims();
	}

	#endregion

	#region Private methods

	/// <summary>
	/// Finds the claimed profiles.
	/// </summary>
	private void FindVerifiedClaims()
	{
		string lookfor = "\"claimed_nodes\":[";
		int begin = Html.IndexOf(lookfor) + lookfor.Length;
		ParseClaims(begin, _VerifiedClaims);
	}

	/// <summary>
	/// Finds the unverified claims.
	/// </summary>
	private void FindUnverifiedClaims()
	{
		string lookfor = "\"unverified_claiming_nodes\":[";
		int firstBegin = Html.IndexOf(lookfor) + lookfor.Length;

		if (firstBegin > lookfor.Length)
		{
			ParseClaims(firstBegin, _UnverifiedClaims);

			int secondBegin = Html.LastIndexOf(lookfor) + lookfor.Length;
			if (secondBegin > firstBegin)
				ParseClaims(secondBegin, _UnverifiedClaims);
		}
	}

	/// <summary>
	/// Parses both the verified and unverified claims.
	/// </summary>
	private void ParseClaims(int begin, StringCollection collection)
	{
		int end = Html.IndexOf("]", begin);
		string match = Html.Substring(begin, end - begin);
		string[] urls = match.Split(',');

		foreach (string s in urls)
		{
			string clean = s.Replace("\"", string.Empty);
			if (!collection.Contains(clean))
				collection.Add(clean);
		}
	}

	/// <summary>
	/// Finds the outbound links.
	/// </summary>
	private void FindOutboundLinks()
	{
		ParseLinks("\"nodes_referenced\":{", _Outbound);
	}

	/// <summary>
	/// Finds the inbound links.
	/// </summary>
	private void FindInboundLinks()
	{
		ParseLinks("\"nodes_referenced_by\":{", _Inbound);
	}

	/// <summary>
	/// Parses both the outbound and the inbound links.
	/// </summary>
	private void ParseLinks(string lookfor, Dictionary<string, StringCollection> dictionary)
	{
		int begin = Html.IndexOf(lookfor) + lookfor.Length;
		int end = Html.IndexOf("}},", begin);

		if (begin > lookfor.Length && end > begin)
		{
			string match = Html.Substring(begin, end - begin);
			if (match.Contains("\"nodes_referenced_by\""))
				return;

			string[] values = match.Split(new string[] { "}," }, StringSplitOptions.None);

			foreach (string value in values)
			{
				string[] triplet = value.Split(new string[] { "\":" }, StringSplitOptions.None);
				string uri = triplet[0].Replace("\"", string.Empty);
				string tags = triplet[2].Replace("[", null).Replace("]", null).Replace("\"", null);

				StringCollection col = new StringCollection();
				string[] xfn = tags.Split(',');
				foreach (string tag in xfn)
				{
					col.Add(tag);
				}

				dictionary.Add(uri, col);
			}
		}
	}

	/// <summary>
	/// Construct the URL of the API endpoint based on whether or not
	/// EnableOutboundLinks and EnableInboundLinks are true or false.
	/// </summary>
	/// <returns></returns>
	private string GetApiEndpoint()
	{
		string baseUrl = "http://socialgraph.apis.google.com/lookup?q={0}&fme=1";

		if (EnableInboundLinks)
			baseUrl += "&edi=1";

		if (EnableOutboundLinks)
			baseUrl += "&edo=1";

		return string.Format(baseUrl, HttpUtility.HtmlEncode(Url.ToString()));
	}

	/// <summary>
	/// Retrieves the HTML from the specified URL.
	/// </summary>
	private void RetrieveHtml()
	{
		string endpoint = GetApiEndpoint();

		using (WebClient client = new WebClient())
		{
			_Html = client.DownloadString(endpoint);
		}
	}

	#endregion

}
